# Generated by Django 2.0.2 on 2018-02-16 15:45

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("automated_logging", "0010_auto_20180216_1430"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="model",
            options={"verbose_name": "Changelog", "verbose_name_plural": "Changelogs"},
        ),
        migrations.AlterModelOptions(
            name="modelchangelog",
            options={},
        ),
        migrations.AlterModelOptions(
            name="request",
            options={"verbose_name": "Request", "verbose_name_plural": "Requests"},
        ),
        migrations.AlterModelOptions(
            name="unspecified",
            options={
                "verbose_name": "Non DJL Message",
                "verbose_name_plural": "Non DJL Messages",
            },
        ),
    ]
