# Generated by Django 3.0.7 on 2020-08-19 10:04

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("automated_logging", "0016_auto_20200803_1917"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="application",
            options={
                "verbose_name": "Application",
                "verbose_name_plural": "Applications",
            },
        ),
    ]
