""" This is just a file that imports all admin interfaces defined in this package """

from automated_logging.admin.model_entry import *
from automated_logging.admin.model_event import *
from automated_logging.admin.model_mirror import *

from automated_logging.admin.request_event import *

from automated_logging.admin.unspecified_event import *
